/*
 * Copyright (C) 2012 - Michael Baudin
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */
 
#ifndef _CDFLIBPRIVATE_H_
#define _CDFLIBPRIVATE_H_

// warning C4996: 'sprintf': This function or variable may be unsafe. 
// Consider using sprintf_s instead. To disable deprecation, 
// use _CRT_SECURE_NO_WARNINGS. See online help for details.
#ifdef _MSC_VER
#pragma warning( disable : 4996 )
#endif

#define min(a,b) ((a) <= (b) ? (a) : (b))
#define max(a,b) ((a) >= (b) ? (a) : (b))
#define TRUE_ (1)
#define FALSE_ (0)

#define Abs(x) ( ( (x) >= 0) ? (x) : -( x) )

#define CDFLIB_MSGFLOATGREQ "%s: Wrong value for input argument ""%s"": Must be >= %e.\n"
#define CDFLIB_MSGFLOATGREATER "%s: Wrong value for input argument ""%s"": Must be > %e.\n"
#define CDFLIB_MSGFLOATLOWEQ "%s: Wrong value for input argument ""%s"": Must be <= %e.\n"
#define CDFLIB_MSGFLOATLOWER "%s: Wrong value for input argument ""%s"": Must be < %e.\n"
#define CDFLIB_MSGFLOATVALUEINT "%s: Wrong value for input argument ""%s"": Must have integer value.\n"
#define CDFLIB_MSGINTGREQ "%s: Wrong value for input argument ""%s"": Must be >= %d.\n"
#define CDFLIB_MSGINTGREATER "%s: Wrong value for input argument ""%s"": Must be > %d.\n"
#define CDFLIB_MSGINTLOWEQ "%s: Wrong value for input argument ""%s"": Must be <= %d.\n"
#define CDFLIB_MSGINTLOWER "%s: Wrong value for input argument ""%s"": Must be < %d.\n"
#define CDFLIB_MSGGENERIC "%s: Wrong value for input argument ""%s"": We must have ""%s"".\n"

// If this function is non-NULL, it is used to print messages out.
// It can be configure with the public function "cdflib_messageSetFunction".
// If the function is not configured, the messages are printed in the console.
extern void (* cdflib_messagefunction)(char * message);

// If this function is non-NULL, it is used to print messages out.
// It can be configure with the public function "cdflib_messageSetFunction".
// If the function is not configured, the messages are printed in the console.
extern void (* cdflib_messagefunction)(char * message);

// Print error messages
void cdflib_messageprint(char * msg);

// The default mode of the verbose (OFF)
static int cdflib_verbose=0;

// Print a "Iteration: %d" message, if verbose mode is enabled.
void cdflib_printiter(char * fname, int iteration);

// Computes ap^bp
double cdflib_powdd(double ap, double bp);

// Returns a double with the same magnitude as a, 
// and the same sign as b.
double cdflib_dsign(double a, double b);

// Rounds a double toward zero.
double cdflib_dint(double x);

/* EVALUATION OF THE FUNCTION LN(1 + A) */
double cdflib_log1p(double a);

// DOUBLE PRECISION MACHINE CONSTANTS
double cdflib_spmpar(int i);

/* INTEGER MACHINE CONSTANTS */
int cdflib_ipmpar(int i);

/* COMPUTATION OF  X - 1 - LN(X) */
double cdflib_rlog(double x);

/* EVALUATION OF THE FUNCTION X - LN(1 + X) */
double cdflib_rlog1(double x);

////////////////////////////////////////////////////////////////
//
// GAMMA-related functions

/* EVALUATION OF THE GAMMA FUNCTION FOR REAL ARGUMENTS */
double cdflib_gamma(double a);

/* EVALUATION OF LN(GAMMA(A)) FOR POSITIVE A */
double cdflib_gamln(double a);

/* Returns LN(GAMMA(X)). */
// TODO : what difference between cdflib_gamln and cdflib_alngam ?
double cdflib_alngam(double x);

/* EVALUATION OF LN(GAMMA(1 + A)) FOR -0.2 .LE. A .LE. 1.25 */
double cdflib_gamln1(double a);

/* COMPUTATION OF LN(GAMMA(B)/GAMMA(A+B)) WHEN B .GE. 8 */
double cdflib_algdiv(double a, double b);

/* EVALUATION OF  DEL(A0) + DEL(B0) - DEL(A0 + B0)  WHERE */
/* LN(GAMMA(A)) = (A - 0.5)*LN(A) - A + 0.5*LN(2*PI) + DEL(A). */
/* IT IS ASSUMED THAT A0 .GE. 8 AND B0 .GE. 8. */
double cdflib_bcorr(double a0, double b0);

/*     EVALUATION OF EXP(-X)*X**A/GAMMA(A) */
// This is X*(derivative of the incomplete Gamma function w.r.t. X)
double cdflib_rcomp(double a, double x);

/* EVALUATION OF THE DIGAMMA FUNCTION */
// This is the derivative of LN(GAMMA(X))
double cdflib_psi1(double xx);

/* EVALUATION OF THE FUNCTION LN(GAMMA(A + B)) */
/* FOR 1 .LE. A .LE. 2  AND  1 .LE. B .LE. 2 */
double cdflib_gsumln(double a, double b);

/* EVALUATION OF THE INCOMPLETE GAMMA RATIO FUNCTIONS */
/*                  P(A,X) AND Q(A,X) */
// These are the regularized Gamma functions
// P(a,x) = [ Int_0^x   t^(a-1)exp(-t)dt ] / gamma(x)
// Q(a,x) = [ Int_x^Inf t^(a-1)exp(-t)dt ] / gamma(x)
/* IT IS ASSUMED THAT A AND X ARE NONNEGATIVE, WHERE A AND X */
/* ARE NOT BOTH 0. */
int cdflib_gratio(double a, double x, double *ans, double *qans, int ind);

/*    EVALUATION OF THE INCOMPLETE GAMMA RATIO FUNCTIONS */
/*                  P(A,X) AND Q(A,X) */
/* IT IS ASSUMED THAT A .LE. 1.  EPS IS THE TOLERANCE TO BE USED. */
/* THE INPUT ARGUMENT R HAS THE VALUE E**(-X)*X**A/GAMMA(A). */
int cdflib_grat1(double a, double x, double r, 
	double *p, double *q, double eps);

/* INVERSE INCOMPLETE GAMMA RATIO FUNCTION */
int cdflib_gaminvfun(double a, double *x, double x0, 
	double p, double q, int *ierr);

/* COMPUTATION OF 1/GAMMA(A+1) - 1  FOR -0.5 .LE. A .LE. 1.5 */
double cdflib_gam1(double a);

//
////////////////////////////////////////////////////////////////

/* EVALUATION OF I (A,B) */
/*                X */
/* FOR B .LT. MIN(EPS,EPS*A) AND X .LE. 0.5. */
double cdflib_fpser(double a, double b, double x, double eps);

/* IF L = 0 THEN  EXPARG(L) = THE LARGEST POSITIVE W FOR WHICH */
/* EXP(W) CAN BE COMPUTED. */
/* IF L IS NONZERO THEN  EXPARG(L) = THE LARGEST NEGATIVE W FOR */
/* WHICH THE COMPUTED VALUE OF EXP(W) IS NONZERO. */
double cdflib_exparg(int l);

/* EVALUATION OF EXP(MU + X) */
double cdflib_esum(int mu, double x);

/* EVALUATION OF THE COMPLEMENTARY ERROR FUNCTION */
/*  ERFC1(IND,X) = ERFC(X)            IF IND = 0 */
/*  ERFC1(IND,X) = EXP(X*X)*ERFC(X)   OTHERWISE */
double cdflib_erfc1(int ind, double x);

/* EVALUATION OF THE REAL ERROR FUNCTION */
double cdflib_erf(double x);

// Solve f(x)=0 equations.
	/*     Double precision ZeRo of a function -- Reverse Communication */
// TODO : make some cleanup here : are all these functions necessary at the public level ?


int cdflib_dstinv(double *zsmall, double *zbig, double 
	*zabsst, double *zrelst, double *zstpmu, double *zabsto, 
	double *zrelto);

/* Double precision NoRmal distribution INVerse */
/* Returns X  such that CUMNOR(X)  =   P,  i.e., the  integral from - */
/* infinity to X of (1/SQRT(2*PI)) EXP(-U*U/2) dU is P */
double cdflib_dinvnr(double p, double q);

/* Initalize Approximation to */
/* INVerse of the cumulative T distribution */
double cdflib_dt1(double p, double q, double df);

/* EVALUATION OF THE FUNCTION EXP(X) - 1 */
double cdflib_expm1(double x);

/* EVALuate a PoLynomial at X */
/*     returns */
/* A(1) + A(2)*X + ... + A(N)*X**(N-1) */
double cdflib_devlpl(double *a, int n, double *x);

////////////////////////////////////////////////////////////////
//
// BETA-related functions
//

/* EVALUATION OF IX(A,B) - IX(A+N,B) WHERE N IS A POSITIVE INTEGER. */
/* EPS IS THE TOLERANCE USED. */
double cdflib_bup(double a, double b, double x, double y, 
	int n, double eps);

/* EVALUATION OF X**A*Y**B/BETA(A,B) */
// If Y=1-X, this is X*Y*(derivative of the incomplete Beta function w.r.t. X)
double cdflib_brcomp(double a, double b, double x, double y);

/* EVALUATION OF  EXP(MU) * (X**A*Y**B/BETA(A,B)) */
double cdflib_brcmp1(int mu, double a, double b, double x, double y);

/* EVALUATION OF THE INCOMPLETE BETA FUNCTION IX(A,B) */
int cdflib_bratio(double a, double b, double x, 
	double y, double *w, double *w1, int *ierr);

/* POWER SERIES EXPANSION FOR EVALUATING 
   INCOMPLETE BETA FUNCTION IX(A,B) WHEN B .LE. 1 */
/* OR B*X .LE. 0.7.  EPS IS THE TOLERANCE USED. */
double cdflib_bpser(double a, double b, double x, double eps);

/* ASYMPTOTIC EXPANSION FOR INCOMPLETE BETA FUNCTION IX(A,B) WHEN A IS LARGER THAN B. */
/* THE RESULT OF THE EXPANSION IS ADDED TO W. IT IS ASSUMED */
/* THAT A .GE. 15 AND B .LE. 1.  EPS IS THE TOLERANCE USED. */
/* IERR IS A VARIABLE THAT REPORTS THE STATUS OF THE RESULTS. */
int cdflib_bgrat(double a, double b, double x, 
	double y, double *w, double eps, int *ierr);

/* CONTINUED FRACTION EXPANSION INCOMPLETE BETA FUNCTION FOR IX(A,B) WHEN A,B .GT. 1. */
/* IT IS ASSUMED THAT  LAMBDA = (A + B)*Y - B. */
double cdflib_bfrac(double a, double b, double x, double y, 
	double lambda, double eps);

/* EVALUATION OF THE LOGARITHM OF THE BETA FUNCTION */
/* LN(BETA(A,B)) */
double cdflib_betaln(double a0, double b0);

/* APSER YIELDS THE INCOMPLETE BETA RATIO I(SUB(1-X))(B,A) FOR */
/* A .LE. MIN(EPS,EPS*B), B*X .LE. 1, AND X .LE. 0.5. USED WHEN */
/* A IS VERY SMALL. USE ONLY IF ABOVE INEQUALITIES ARE SATISFIED. */
double cdflib_apser(double a, double b, double x, double eps);

/* ASYMPTOTIC EXPANSION FOR IX(A,B) FOR LARGE A AND B. */
/* LAMBDA = (A + B)*Y - B  AND EPS IS THE TOLERANCE USED. */
/* IT IS ASSUMED THAT LAMBDA IS NONNEGATIVE AND THAT */
/* A AND B ARE GREATER THAN OR EQUAL TO 15. */
double cdflib_basym(double a, double b, double lambda, double eps);

//
////////////////////////////////////////////////////////////////

/* STarting VALue for Neton-Raphon */
/* calculation of Normal distribution Inverse */
double cdflib_stvaln(double p);

/* Computes A**N - 1, THE LARGEST MAGNITUDE where */
/* A, THE BASE. */
/* N, THE NUMBER OF BASE-A DIGITS. */
int cdflib_largestint(void);

/////////////////////////////////////////////////////
//
// Floating point parameters
//

// Returns the relative tolerance.
// This is twice the machine epsilon.
// i.e. ~2.e-16.
double cdflib_doubleEps(void);

// Returns the absolute tolerance.
// This is the underflow threshold.
// i.e. 1.e-308.
double cdflib_doubleTiny(void);

// This is the overflow threshold.
// i.e. 1.e308.
double cdflib_doubleHuge(void);

// This is the positive infinite, i.e. INF.
double cdflib_infinite(void);

// Returns a Nan.
double cdflib_nan();

// Returns 0 if x is a NAN.
double cdflib_isnan(double x);

// Checks the value of an integer parameter.
//
// Arguments
// fname : the name of the function
// param : the parameter
// paramname : the name of the parameter
// minparam : the minimum value of the parameter
// maxparam : the maximum value of the parameter
//
// Description
// Checks that param is in the range 
// {minparam, minparam+1, ..., maxparam}.
// If this is OK, then returns CDFLIB_OK.
// If not returns CDFLIB_ERROR.
int cdflib_checkrangeint(char * fname, int param, 
	char * paramname, int minparam, int maxparam);

// Checks the value of a double parameter.
//
// Description
// Checks that param>=minparam and param<=maxparam. 
// Otherwise, this is the same as cdflib_checkrangeint.
int cdflib_checkrangedouble(char * fname, double param, 
	char * paramname, double minparam, double maxparam);

// Checks the value of p
//
// Arguments
// fname : the name of the function
// p : the p-value
// pname : the name of p
//
// Description
// Checks that p in [0,1].
// If this is OK, then returns CDFLIB_OK.
// If not returns CDFLIB_ERROR.
int cdflib_checkp(char * fname, double p, char * pname);

// Checks the values of p and q
//
// Arguments
// fname : the name of the function
// p : the p-value
// pname : the name of p
// q : mathematically equal to 1-p, but may be numerically different if p is close to 1
// qname : the name of q
//
// Description
// Checks that p in [0,1], and q in [0,1], and p+q==1 and p+q>0.
// More precisely, checks that abs(p+q-1)<=3*eps, where 
// eps is the machine epsilon. 
// If this is OK, then returns CDFLIB_OK.
// If not returns CDFLIB_ERROR.
int cdflib_checkpq(char * fname, double p, char * pname, double q, char * qname);

// Checks that x+y==1 and x+y>0.
//
// Arguments
// fname : the name of the function
// x : the value of x
// y : the value of y
// paramname : the name of the formal sum "x+y"
//
// Description
// More precisely, checks that abs(x+y-1)<=3*eps, where 
// eps is the machine epsilon. 
// If this is OK, then returns CDFLIB_OK.
// If not returns CDFLIB_ERROR.
int cdflib_checksumtoone(char * fname, double x, double y, char * paramname);

// Checks that value>minparam (double)
//
// Arguments
// fname : the name of the function
// param : the parameter
// paramname : the name of the parameter
// minparam : the lowerbound
//
// Description
// If this is OK, then returns CDFLIB_OK.
// If not returns CDFLIB_ERROR.
int cdflib_checkgreaterthan(char * fname, double param, 
	char * paramname, double minparam);

// Checks that value>minparam (int)
int cdflib_checkintgreaterthan(char * fname, int param, 
	char * paramname, int minparam);

// Checks that value>=minparam (double)
//
// Arguments
// fname : the name of the function
// param : the parameter
// paramname : the name of the parameter
// minparam : the lower bound
//
// Description
// If this is OK, then returns CDFLIB_OK.
// If not returns CDFLIB_ERROR.
int cdflib_checkgreqthan(char * fname, double param, 
	char * paramname, double minparam);

// Checks that value>=minparam (int)
int cdflib_checkintgreqthan(char * fname, int param, 
	char * paramname, int minparam);

// Checks that value<=maxparam.
//
// Arguments
// fname : the name of the function
// param : the parameter
// paramname : the name of the parameter
// maxparam : the upper bound
//
// Description
// If this is OK, then returns CDFLIB_OK.
// If not returns CDFLIB_ERROR.
int cdflib_checkloweqthan(char * fname, double param, 
	char * paramname, double maxparam);
int cdflib_checkintloweqthan(char * fname, int param, 
	char * paramname, int maxparam);

// Computes the zero function for inverse CDF computation.
//
// Arguments
// p : the probability
// q : the complementary probability (mathematically equal to 1-p)
// cum : the value of P(X<x)
// ccum : the value of P(X>x)
//
// Description
// If p<q, then computes cum-p.
// If not, then computes ccum-q.
double cdflib_computefx(double p, double q, double cum, double ccum);

//   Computes the probability distribution function of 
//   the Binomial distribution.
//
// Arguments
// x : the number of successes 
// n : the number of Bernouilli experiments
// p : the probability of success
// q : the probability of failure (mathematically, we have q=1-p)
// computelog : if computelog==CDFLIB_LOGCOMPUTE, then returns log(y), otherwise returns y.
//
// Description
// Computes
//
//f(x) = nchoosek(n,x)* p^x * q^(n-x)
//
// Returns the probability that X=x.
//
// Allows to have a non-integer value of x and N. 
//
// The implementation uses the following principles.
//
// We have:
// beta(x+1,n-x+1)=gamma(x+1)*gamma(n-x+1)/gamma(n+2)
// But gamma(n+2)=(n+1)*gamma(n)
// which implies
// beta(x+1,n-x+1)=gamma(x+1)*gamma(n-x+1)/(n+1)/gamma(n+1)
// Therefore:
// gamma(x+1)*gamma(n-x+1)/gamma(n+1) = (n+1)*beta(x+1,n-x+1)
//
// Hence,
//
// nchoosek(n,x)=1/(n+1)/beta(x+1,n-x+1)
//
// All in all, we have :
// f(x)= p^x*(1-p)^(n-x)/beta(x+1,n-x+1)/(n+1)
// This can be written:
// f(x)= p^(x+1)*(1-p)^(n-x+1)/beta(x+1,n-x+1)/(n+1)/p/(1-p)
// Let a=x+1 and b=n-x+1. This is:
// f(x)= p^a*(1-p)^b/beta(a,b)/(n+1)/p/(1-p)
// So we can use cdflib_brcomp which computes:
// t=X**A*Y**B/BETA(A,B)
// with X=p, Y=1-p, A=a, B=b.
// Then, f(x)= t/(n+1)/p/(1-p).
// 
// Bibliography
// http://svn.r-project.org/R/trunk/src/nmath/dbinom.c
double cdflib_binopdfraw(double x,double n,double p,double q, int computelog);

// Print a -Unable to invert message-
//
// Arguments
// fname : the function name
// bound : the achieved bound
// paramname : the parameter which was searched for
//
// Description
// Just prints a message.
void cdflib_unabletoinvert(char * fname, double bound, char * paramname);

// Check the tail option of a function
//
// Arguments
// fname : the function name
// lowertail : the tail option
//
// Description
// Produces an error if lowertail is both different from 
// CDFLIB_UPPERTAIL and CDFLIB_LOWTAIL.
int cdflib_checklowertail(char * fname, int lowertail);

// Check that a double has an integer value.
//
// Arguments
// fname : the function name
// x : the value
// paramname : the name of the parameter
//
// Description
// Produces an error if x has a nonzero fractional part, i.e. 
// has not an integer value.
int cdflib_checkIntValue(char * fname, double x, char * paramname);

//////////////////////////////////////////////////////////
// 
// F Distribution
//
int cdflib_fCheckParams(char * fname, double v1, double v2);
int cdflib_fCheckX(char * fname, double x);
int cdflib_cumf(double x, double v1, double v2, double *cum, double *ccum);

//////////////////////////////////////////////////////////
// 
// Chi-square Distribution
//
int cdflib_chiCheckParams(char * fname, double df);
int cdflib_chiCheckX(char * fname, double x);
int cdflib_cumchi(double x, double df, double *cum, double *ccum);

//////////////////////////////////////////////////////////
// 
// Normal Distribution
//
int cdflib_norCheckParams(char * fname, double mean, double sd);
int cdflib_norCheckX(char * fname, double x);
int cdflib_cumnor(double x, double *cum, double *ccum);

//////////////////////////////////////////////////////////
// 
// Beta Distribution
//
int cdflib_betCheckParams(char* fname, double a, double b);
int cdflib_betCheckXY(char* fname, double x, double y);
int cdflib_cumbet(double x, double y, double a, double b, double *cum, double *ccum);
int cdflib_betCheckX(char* fname, double x);

//////////////////////////////////////////////////////////
// 
// Non-Central Chi-Square Distribution
//
int cdflib_chnCheckParams(char * fname, double df, double pnonc);
int cdflib_chnCheckX(char * fname, double x);
int cdflib_cumchn(double x, double df, double pnonc,double *cum, double *ccum);

//////////////////////////////////////////////////////////
// 
// Binomial Distribution
//
int cdflib_binCheckParams(char * fname, double n, double pr);
int cdflib_binCheckX(char * fname, double n, double x);
int cdflib_cumbin(double x, double xn, double pr, double ompr, double *cum, double *ccum);

//////////////////////////////////////////////////////////
// 
// Negative Binomial Distribution
//
int cdflib_nbnCheckX(char * fname, double x);
int cdflib_nbnCheckParams(char * fname, double n, double pr, double ompr);
int cdflib_cumnbn(double x, double xn, double pr, double ompr, double *cum, double *ccum);

//////////////////////////////////////////////////////////
// 
// T Distribution
//
int cdflib_tCheckX(char * fname, double x);
int cdflib_tCheckParams(char * fname, double v);
int cdflib_cumt(double x, double v, double *cum, double *ccum);

//////////////////////////////////////////////////////////
// 
// Poisson Distribution
//
int cdflib_cumpoi(double x, double xlam, double *cum, double *ccum);

//////////////////////////////////////////////////////////
// 
// Non-Central F Distribution
//
int cdflib_fncCheckX(char * fname, double x);
int cdflib_fncCheckParams(char * fname, double dfn, double dfd, double phonc);
int cdflib_cumfnc(double x, double dfn, double dfd, double pnonc, double *cum, double *ccum);

//////////////////////////////////////////////////////////
// 
// Gamma Distribution
//
int cdflib_gamCheckParams(char * fname, double shape, double rate);
int cdflib_gamCheckX(char * fname, double x);
int cdflib_cumgam(double x, double a, double *cum, double *ccum);

// If this function is non-NULL, it is used to generate uniform random numbers in [0,1).
// It can be configure with the public function "cdflib_randSetFunction".
extern double (* cdflib_randfunction)();

// If this function is non-NULL, it is used to generate uniform random integers in [a,b].
// It can be configure with the public function "cdflib_randIntegerInRangeSetFunction".
extern double (* cdflib_randIntegerInRange)(double a, double b);

// If this function is non-NULL, it is used to print messages out.
// It can be configure with the public function "unifrng_msgsetfunction".
// If the function is not configured, the messages are printed in the console.
extern void (* cdflib_messagefunction)(char * message);

// The function which generates random numbers uniform in [0,1).
double cdflib_randgenerate();

// The function which generates random integers uniform in [a,b].
double cdflib_generateIntegerInRange(double a, double b);

// Standard Exponential Random Variable
double cdflib_sexpo ();

// Standard Gamma Random Variable
double cdflib_sgamma (double a);

// Generate Standard Normal Random Variables
double cdflib_snorm();

//
// Elementary Functions
//

// Computes log(1+x) accurately for small x
double cdflib_logp1(double x);

//  Compute   v = log ( (1 + s)/(1 - s) )  for small s, this is for |s| < SLIM = 0.20
double cdflib_lnp1m1(double s);

//
// Numerical Linear Algebra
//

// Scalar product of vectors sx and sy
double cdflib_sdot (int *n, double *sx, int *incx, double *sy, int *incy);

// Factors a symmetric positive definite matrix
int cdflib_spofa (double *a, int *lda, int *n, int *info);

#endif /** _CDFLIBPRIVATE_H_   **/
