// =============================================================================

// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2010-2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// =============================================================================
// <-- JVM NOT MANDATORY -->

path = distfun_getpath (  );
exec(fullfile(path,"tests","unit_tests","testingutilities.sce"));

rtol = 1.e-2;

// Set the seed to always get the same random numbers
distfun_seedset(0);

//
// 1. Permute some vectors, and check that the output is basically correct.
//
X = (2:10)';
P = distfun_rndprm(1,X);
assert_checkequal ( typeof(P) , "constant" );
assert_checkequal ( size(P) , size(X) );
assert_checkequal ( gsort(P,"g","i") , X );
//
X = (10:100)';
P = distfun_rndprm(1,X);
assert_checkequal ( typeof(P) , "constant" );
assert_checkequal ( size(P) , size(X) );
assert_checkequal ( gsort(P,"g","i") , X );
//
X = [-12 4 9 365]';
P = distfun_rndprm(1,X);
assert_checkequal ( typeof(P) , "constant" );
assert_checkequal ( size(P) , size(X) );
assert_checkequal ( gsort(P,"g","i") , X );
//
X = (2:11)';
P = distfun_rndprm(5,X);
assert_checkequal ( typeof(P) , "constant" );
assert_checkequal ( size(P) , [10 5] );
assert_checkequal ( gsort(P,"r","i") , X * ones(1,5) );
//
// 2. Check that the produced permutations are uniformly chosen in the 
// set of all possible permutations.
//

X = [-12 4 9 365]';
S = perms(X);
// Total number of permutations : F = 4! = 24
F = size(S,"r");
// Number of random permutations : N
N = 10000;
P = distfun_rndprm(N,X);
P = P';
// Set R(k)=i, k=1,2,...,N the index i of the permutation, with i=1,2,...,F.
// R must be a random variable uniformly distributed in the interval [1,2,...,F].
// In other words, the index i of the permutation #k has been uniformly 
// chosen in the set of possible permutations [1,2,...,F].
R = zeros(N,1);
for k = 1:N
  permk = P(k,:);
  // Search the index i of the permutation permk in S
  // i.e. search i such that S(i,:)==permk
  i = vectorfind(S,permk,"r");
  R(k) = i;
end

checkMeanVariance ( N , 1 , R , (1+F)/2 , (F^2-1)/12, 0.1 );
function p = mycdfuin (X,A,B)
  p = (floor(X)-A+1)/(B-A+1)
endfunction
NC = 2*12;
A = 1;
B = F;
rtol = 0.01;
atol = 0.01;
checkRNGLaw ( R , list(mycdfuin,A,B) , N , NC , rtol , atol , %f );
